(function () {
    'use strict';
    angular
        .module('appClient')
        .directive('atCrudForm', atCrudForm);

    /** @ngInject */
    function atCrudForm() {
        var bindings = {
            options: '=',
            resource: '=',
            active: '@'
        };
        var directive = {
            restrict:'A',
            scope:true,
            link: link,
            controller: AtCrudFormController,
            controllerAs: 'vm',
            bindToController: bindings
        };
        return directive;
    };

    /** @ngInject */
    function link(scope, element, attrs) {
        scope.positionValdrMessages = function(){
            angular.element(element).find('.valdr-message').each(function(i){
                var field = angular.element(this).attr('valdr-message');
                var $fieldElement;
                angular.element(element).find('[name='+field+']').each(function(){
                    $fieldElement = angular.element(this);
                });
                if($fieldElement != undefined){
                    if($fieldElement.parent().hasClass('select-style')) {
                        $fieldElement.parent().after(angular.element(this));
                    }else{
                        $fieldElement.after(angular.element(this));
                    }
                }
            });
        };
        scope.positionValdrMessages();
        angular.element(element).addClass('at-crud-form');
    }

    /** @ngInject */
    function AtCrudFormController($scope,$element,$attrs,$filter,$location,$timeout,Api,AppService,AppSettings,ViewService,toastr) {
        var vm = this;
        vm.id = AppService.getUUID();
        vm.idField = AppSettings.idField;
        vm.editEnabled = true;
        vm.state = undefined;
        vm.visible = true;
        vm.api = Api;
        $scope.submit = submit;
        $scope.lovs = Api.getLovItems;
        

        if(!vm.options){
            init();
        }

        /*
        // event listeners
        */
        $scope.$on('crudButtonClick',crudButtonClickHandler);
        $scope.$on('permissionChange',applyFieldPermissions);

        $scope.$on('resource-loaded',function(event,data){
            if(data.resource === vm.resource){
                applyState();
            }
        });
        $scope.$on('create-resource',function(event,data){
            if(data.resource === vm.resource){
                applyState();
            }
        });
        $scope.$on('formError',function(event,data){
            vm.isSubmitting = false;
        });

        /*
        // watchers
        */
        $scope.$watch('vm.options',function(){
            if(!vm.options || vm.ready){return}
            vm.ready = true;
            initCrudToolbar();
            init();
        });
        $scope.$watch('vm.resource',function(){
            if(!vm.resource){return;}
            $scope.resource = vm.resource;
            if(vm.resource._session && vm.resource._session.state !== vm.state){
                vm.state = vm.resource._session.state;
            }
            applyFieldPermissions();
        });
        $scope.$watch('vm.state',function(){
            if(!vm.state){return}
            applyState();
        });
        $scope.$watch('vm.enabled',function(){
            if(!vm.enabled){return}
            applyState();
        });

        function init(){
            if(vm.initialized){ return }
            vm.initialized = true;
            show();
            if(!$attrs.id){
                $attrs.id = vm.id;
                angular.element($element).attr('id',vm.id);
            }
            AppService.entities[$attrs.id] = { bundle:'at-crud-form', id:vm.id, form:$scope.form,vm:vm };
            vm.isSubmitting = false;
            // standard form init
            if(!vm.options){
                vm.editEnabled = true;
                vm.state = 'edit';
            }
            if (vm.options) {
                if (vm.options.idField) {
                    vm.idField = vm.options.idField;
                }
                if(vm.options.bundle){
                    vm.bundle = vm.options.bundle;
                }
                if(vm.options.editEnabled === false){
                    vm.editEnabled = vm.options.editEnabled;
                }
            }
            resetComponents();
        }

        function submit() {
            if( !$scope.form || vm.isSubmitting || vm.active === 'false'){return}
            if(AppSettings.dev){
                console.log('submit');
                console.log($scope.form);
            }
            $scope.form.$setSubmitted(true);
            if (!$scope.form.$valid) {
                // invalid form
                vm.isSubmitting = true;
                if(AppSettings.dev){
                    console.log('form.$valid='+$scope.form.$valid);
                    angular.forEach($scope.form.$error.valdr,function(v,i){
                        console.log('validation error '+i+ ' = '+v.$name);
                    });
                }
                $timeout(function(){
                    vm.isSubmitting = false;
                },100);
                toastr.warning($filter('translate')('FORM INVALID')+'.');
            }else{
                // valid form
                if($scope.resource) {
                    vm.isSubmitting = true;
                    saveResource($scope.resource);
                    AppService.scope.$broadcast('submit', {id: $attrs.id, form:$scope.form, resource:$scope.resource, vm:vm})
                }else{
                    vm.isSubmitting = false;
                    AppService.scope.$broadcast('submit',{id:$attrs.id,form:$scope.form,vm:vm,scope:$scope})
                }
            }
        }

        function reset(id,bundle){
            if(vm.active === 'false'){return;}
            if (vm.options && vm.options.vm && vm.options.vm.gridView) {
                if(AppSettings.dev){
                    console.log('reset:'+bundle+': '+id);
                }
                if (id && bundle) {
                    // on put + post
                    var params = {};
                    params[bundle] = id;
                    if (vm.options.bundleParent) {
                        params[vm.options.bundleParent] = vm.options.idParent;
                    }
                    $location.search(params);
                } else {
                    // on delete + cancel create
                }
                hide();
                $scope.resource = vm.resource = vm.options.vm.resource = vm.options.vm.gridView.resource = undefined;
                $timeout(vm.options.vm.init);
            }
        }

        function crudButtonClickHandler(event,data){
            if(vm.resource && data.ID && data.ID === vm.resource.ID){
                // edit
                if(data.action==='edit'){
                    if(vm.resource._session) {
                        vm.state = vm.resource._session.state = 'edit';
                    }else{
                        vm.state = 'edit';
                    }
                    applyState();
                }
                // put
                if(data.action==='save'){
                    submit();
                }
                // delete
                if(data.action==='delete'){
                    deleteResource();
                }
            }
            // post
            if(data.action ==='save' && !data.ID && data.resource === vm.resource){
                submit();
            }
            // reset
            if(data.action === 'reset' && data.resource === vm.resource){
                if(vm.resource.ID) {
                    reset(vm.resource.ID,vm.resource.bundle);
                }else{
                    reset();
                }
            }
        }

        function saveResource(resource){
            if(vm.active === 'false'){return}
            if(resource[vm.idField]) {
                // PUT
                Api.putResource(resource, vm.idField).then(onResourceSaved);
            } else {
                // POST
                var baseResource;
                if(vm.options.baseResource) {
                    baseResource = vm.options.baseResource;
                } else {
                    baseResource = vm.options.vm.gridView.baseResource;
                }
                Api.postResource(baseResource,resource).then(onResourceSaved);
            }
        }

        function onResourceSaved(resource){
            AppSettings.currentForm = undefined;
            toastr.success($filter('translate')('FORM SUCCESS') + '.');
            vm.isSubmitting = false;
            reset(resource[vm.idField],resource.bundle);
        }

        function createResource() {
            ViewService.checkFormChanged().then(function(bool){
                if(bool === false) {
                    var bundle = vm.options.bundle;
                    vm.resource = $scope.resource = undefined;
                    Api.createResource(bundle).then(function(res){
                        vm.resource = $scope.resource = res;
                        if(vm.options && vm.options.vm && vm.options.vm.resource){
                            vm.options.vm.resource = res;
                        }
                        // set secondary resource parent ID
                        if(vm.options && vm.options.vm && vm.options.vm.gridView) {
                            vm.options.vm.gridView.resource = res;
                            if (vm.options.bundleParent) {
                                res[vm.options.bundleParent + 'ID'] = vm.options.idParent; // todo: ??
                            }
                            if (vm.options.parentSelected) {
                                res[vm.options.parentSelected + 'ID'] = vm.options.idParent;
                            }
                        }
                        vm.state = res._session.state;
                    });
                } else {
                    return;
                }
            })
        }

        function deleteResource(){
            if( vm.isSubmitting || vm.active === 'false'){return}
            vm.isSubmitting = true;
            $timeout(function(){
                vm.isSubmitting = false;
            },100);
            console.log('delete:'+vm.id);
            Api.deleteResource(vm.resource).then(function(res){
                console.log('delete success');
                console.log(res);
                reset();
                toastr.success($filter('translate')('FORM SUCCESS')+'.');

            }).catch(Api.errorHandler);
        }

        function applyState(){
            if(vm.active === 'false'){return}
            show();
            AppSettings.currentForm = undefined;
            // console.log('----vm.state='+vm.state);
            if(vm.state === 'view'){
                setFormEnabled(vm.editEnabled);
                ViewService.scrollToElement(vm.id);
            }
            if(vm.state === 'edit'){
                $scope.editForm = true;
                setFormEnabled(true);
                AppSettings.currentForm = $scope.form;
                ViewService.scrollToElement(vm.id);
            }
            if(vm.state === 'create'){
                $scope.newForm = true;
                setFormEnabled(true);
                AppSettings.currentForm = $scope.form;
                ViewService.scrollToElement(vm.id);
            }
        }

        function setFormEnabled(boolean){
            if(boolean === true) {
                enableForm(angular.element($element));
            }else{
                disableForm(angular.element($element));
            }
        }

        function enableForm($form){
            if(vm.active === 'false'){return}
            //console.log('enableForm: '+vm.id);
            $form.$submitted = false;

            $form.find(":input:not(.always-disabled)").prop('readonly', false);
            $form.find(":input:not(.always-disabled)").prop('disabled', false);
            $form.find("select:not(.always-disabled)").prop('disabled', false);
            $form.find("multiselect:not(.always-disabled)").attr('disabled', false);
            $form.find("at-datepicker:not(.always-disabled)").attr('disabled', false);

            applyFieldPermissions();

            angular.element($form).find('at-crud-toolbar button').each(function(){
                angular.element(this).prop('disabled', false);
            });

            $scope.newForm = undefined;
            $scope.editForm = undefined;

            initFormElements();
            resetComponents();
        }

        function disableForm($form){
            if(vm.active === 'false'){return}
            //console.log('disableForm: '+vm.id);

            $form.find(":input").prop('readonly', true);
            $form.find(":input").prop('disabled', true);
            $form.find("select").prop('disabled', true);
            $form.find("multiselect").attr('disabled', true);
            $form.find("at-datepicker").attr('disabled', true);

            applyFieldPermissions();

            angular.element($form).find('at-crud-toolbar button').each(function (){
                angular.element(this).prop('disabled', false);
                angular.element(this).prop('readonly', false);
            });

            angular.element($form).find('[at-crud]').each(function(i){
                var $el = angular.element(this);
                var crud = Api.getPermission($el.attr('at-crud'));
            });
            resetComponents();
        }

        function applyFieldPermissions(){
            if(vm.active === 'false'){return}
            //console.log('applyFieldPermissions: '+vm.bundle);
            if(vm.bundle){
                angular.element($element).find("[ng-model]").each(function(i){
                    var bundle = vm.bundle;
                    var arr =  angular.element(this).attr('ng-model').split('.');
                    var field = arr[arr.length-1];
                    var crud = Api.getPermission(bundle+'.'+field);
                    //console.log(field);
                    //console.log(crud);
                    var $this = angular.element(this);
                    // no read
                    if(crud[1]===false){
                        $this.closest('div[valdr-form-group]').hide();
                    }
                    // no update + no delete
                    if(crud[2]===false || crud[3]===false){
                        $this.prop('readonly', true);
                        $this.prop('disabled', true);
                    }
                    // update + no delete
                    if(crud[2]===true && crud[3]===false && $scope.editForm){
                        $this.prop('readonly', false);
                        $this.prop('disabled', false);
                    }
                    // create
                    if(crud[0]===true && $scope.newForm === true){
                        if(!$this.hasClass('always-disabled')){
                            $this.prop('readonly', false);
                            $this.prop('disabled', false);
                        };
                    }
                });
            }
        }

        function initFormElements(){
            if(vm.formInitialized){return}
            vm.formInitialized = true;
            angular.element($element).find("select").on('change', function(){
                $scope.$emit('select-change',{resource:$scope.resource,element:angular.element(this),model:angular.element(this).attr('ng-model')})
            });
        }

        function initCrudToolbar(){
            angular.element($element).find('at-crud-toolbar button').each(function(i){
                angular.element(this).prop('disabled',false);
                if(angular.element(this).hasClass('btn-new')){
                    angular.element(this).on('click', function (event) {
                        event.preventDefault();
                        createResource();
                    })
                }
            });
        }

        function hide(){
            angular.element($element).hide();
            vm.visible = false;
        }
        function show(){
            angular.element($element).show();
            vm.visible = true;
        }

        function resetComponents(){
            $timeout($scope.positionValdrMessages,100);
            angular.element($element).find('at-crud-toolbar button').each(function(i) {
               // angular.element(this).prop('disabled', false);
            });
            // todo: date-picker
            angular.element($element).find('[uib-datepicker-popup]').each(function(){
                initDatePicker(angular.element(this));
            });
        }








        // todo: dataPicker
        function initDatePicker($el){
            console.log($el);
            console.log($el.attr('ng-model'));
            var modelId = $el.attr('ng-model');
            var options = {
                modelId: modelId,
                popUpId: 'pop-'+modelId,
            };

            $el.attr('uib-datepicker-popup',AppSettings.dateFormatPicker);

            $scope.open1 = function() {
                $scope.popup1.opened = true;
            };

            $scope.popup1 = {
                opened: false
            };
           console.log(options);
            // uib-datepicker-popup="'dd-MM-yyyy'"
            // ng-model="resource.beginDatum"
            // is-open="popup1.opened"
            // min-date="minDate"
            // max-date="maxDate"
            // datepicker-options="dateOptions"
            // date-disabled="disabled(date, mode)"
            // ng-required="true"
            // close-text="Close"
            // alt-input-formats="altInputFormats" />
        }

    }
})();
