(function () {
    'use strict';
    angular
        .module('appClient')
        .directive('atCrudList', atCrudList)

    /** @ngInject */
    function atCrudList($animate,$interval,$timeout,Api,ViewService,$compile) {
        var bindings = {
            options: '='
        };
        var directive = {
            restrict: 'E',
            scope: true,
            controller: AtCrudListController,
            controllerAs: 'vm',
            bindToController: bindings,
            template: function(tElem, tAttrs){
                var url = tAttrs.template || 'app/components/app-base/directives/at-crud-list/templates/at-crud-table/at-crud-table.html';
                return '<at-crud-table template="'+url+'" ng-if="ready" list="collection.fl" ct="collection.ct"></at-crud-table>';
            },
            link: function(scope, element, attrs) {
                scope.fixPaginationLinks(element);
            }
        };
        return directive;
    };

    /** @ngInject */
    function AtCrudListController($scope,$element,$attrs,$interval,$confirm,$q,$filter,$timeout,Api,AppService,AppSettings,ViewService,Restangular,toastr) {
        var vm = this;
        $scope.collection = {
            fl:{},
            ct:{}
        };
        this.init = function(event){
            if(!vm.options){return;}
            initFlexiList(vm.options);
        };
        $scope.$watch('vm.options',this.init);
        $scope.fixPaginationLinks = fixPaginationLinks;

        function initFlexiList(options){
            vm.options = options;

            // flexilist options
            var flexiListDefaults = {
                method: 'GET',
                urlencoded: true,
                pagesize: 10,
                sortOnClient: true,
                paginationOnClient: true,
                searchOnClient: true,
                selectEnabled: false,
                // Enable log
                //,log: {id: 'ctPhonesExCtrl', err: true, debug: true}
            };
            // override defaults
            angular.extend(flexiListDefaults, options.listConfig);

            // crudTable options
            var crudTableDefaults = {
                pk: 'ID',
                refreshBtn: false,
                actionsCol: true,
                pagesizes: [5,10,20,40,80,160],
                debug: true,
                selectAllOnLoad: false,
                addRecord: createResource,
                editRecord: editResource,
                deleteRecord: deleteRecource,
                deleteRows: deleteRows,
                displayField: function(scope, element, attrs) {
                    element.html(attrs['value']);
                },
                alterResource: undefined,
                enableSearch: true
            };
            // override defaults
            angular.extend(crudTableDefaults, options.listConfig);

            $scope.collection.fl.options = flexiListDefaults;
            $scope.collection.ct.options = crudTableDefaults;
            /*
            // init crudTable
             */
            $scope.ready = true;
            fixPaginationLinks();

            /*
             // Optionally set an initial search for a specific phone
             $scope.collection.ct.search_field = $scope.collection.ct.options.filters[0];
             $scope.collection.ct.search_value = 'Dell Venue';

             $scope.collection.fl.options.where = [{
             field: $scope.collection.ct.search_field.field,
             condition: $scope.collection.ct.search_field.condition,
             value: $scope.collection.ct.search_value,
             options: $scope.collection.ct.search_field.options
             }];
             */

            $scope.$on('flStartOp', function($e, $args){
                if($args.op === 'getPage'){
                    fixPaginationLinks($element);
                }
                if($scope.collection.ct.options.selectAllOnLoad){
                    $scope.collection.fl.toggleSelectAll();
                }
                //$scope.collection.ct.busy = true;
                // $scope.collection.fl.loadData();
            });

            $scope.$on('flComplete', function($e,$args) {
                // toggle all selection
                if($scope.collection.ct.options.selectAllOnLoad){
                    $timeout($scope.collection.fl.toggleSelectAll,200)
                }
                // alter records callback
                angular.forEach($scope.collection.fl.records, function (record) {
                    if($scope.collection.ct.options.alterResource){
                        $scope.collection.ct.options.alterResource(record);
                    }
                });
            });
        }

        function editResource(resource){
                if(!resource._session){
                    resource._session = {state:'edit'}
                }
                ViewService.openModal({template:AppSettings.path_to_templates+'entity-modal.html',data:{resource:resource,options:vm.options},size:'lg'});
               // ViewService.openModal({template:AppSettings.path_to_templates+'entity-modal.html',data:{resource:resource,options:options},size:'lg'},controller:vm.modalController);
        }
        function deleteRecource(resource){
            $confirm({text: $filter('translate')('CONFIRM DELETE')+'?', title: $filter('translate')('DELETE'), ok: $filter('translate')('YES'), cancel: $filter('translate')('NO')})
                .then(function() {
                    Api.deleteResource(resource).then(function(){
                        console.log('delete succes');
                    });
                    console.log(record);
                    console.log('OK');
                });
        }
        function deleteRows(selection){
            function deleteMultiple(){
                var i = 0
                angular.forEach(selection,function(resource){
                    // todo:
                    //Api.deleteResource(resource).then(function(){
                    //    console.log('delete succes');
                    if(i===selection.length-1){ console.log('done:'+i) }
                    //});
                    i++;
                    console.log(resource);
                })
            }
            $confirm({text: $filter('translate')('CONFIRM DELETE MULTIPLE')+'?', title: $filter('translate')('DELETE'), ok: $filter('translate')('YES'), cancel: $filter('translate')('NO')})
                .then(function() {
                    console.log('OK');
                    deleteMultiple();
                });
        }
        function createResource() {
            var bundle = vm.options.bundle;
            console.log('createResource: '+ bundle);
            Api.createResource(bundle).then(function(res){
                vm.resource = res;
                vm.state = res._session.state;
                console.log(res);
                editResource(res)
            });
        }
        function fixPaginationLinks($el){
            $timeout(function(){
                angular.element($el).find('.pagination li a').each(function(el,i){
                    angular.element(this).removeAttr('href');
                })
            },2000)
        }
    }
})();
