(function () {
    'use strict';
    angular
        .module('appClient')
        .directive('atCrudToolbar', atCrudToolbar);

    /** @ngInject */
    function atCrudToolbar($animate,Api,AppService) {
        var bindings = {
            controls: "=",
            resource: "=",
            options: "=",
            bundle: '@',
            //onNew: '=',
            //onEdit: '=',
            //onSave: '=',
            //onDelete: '=',
            //onReset: '='
        };
        var directive = {
            templateUrl: function(elem, attr){
                var url = attr.template || 'app/components/app-base/directives/at-crud-toolbar/at-crud-toolbar.html';
                return url;
            },
            scope: true,
            controller: AtCrudToolbarController,
            controllerAs: 'vm',
            bindToController: bindings,
            link: function(scope, element, attrs) {
                if(scope.vm.bundle){
                    scope.applyCrudPermissions(scope.vm.bundle);
                }
                scope.render();
            }
        };
        return directive;
    }

    /** @ngInject */
    function AtCrudToolbarController($scope,$attrs,$element,Api,AppService,ViewService) {
        var vm = this;
        vm.elements = {};
        $scope.vm = vm;

        vm.crud = [true,true,true,true];
        vm.state = 'view';

        vm.init = function(){
            if(vm.resource.bundle) {
                vm.bundle = vm.resource.bundle;
                vm.ID = vm.resource.ID
            }
            //console.log(vm);
            $scope.applyCrudPermissions(vm.bundle);
        };

        $scope.$watch('vm.controls',function(){
            if(!vm.controls){return};
            $scope.render();
        });

       // $scope.$watch('vm.resource',function(){
        $scope.$watch('vm.resource._session.state',function(){
            if(!vm.resource){return};
            if(!vm.resource._session){
                vm.resource._session = {state:'edit'};
            }
            if(vm.resource._session && vm.resource._session.state){
                 vm.state = vm.resource._session.state;
            }
            vm.init();
        });
        $scope.$on('permissionChange',function(){
            $scope.applyCrudPermissions(vm.bundle);
        });

        $scope.applyCrudPermissions = function(bundle){
            vm.crud = Api.getPermission(bundle);
            //console.log(bundle);
            //console.log(vm.crud);
        };

        $scope.render = function(bundle){
            if(vm.initialized){return;}
            vm.initialized = true;

            angular.element($element).find('li').hide();

            var buttons = [];

            //  new: at-entity controls attr binding
            if(vm.controls && vm.controls.length) {
                buttons = vm.controls;
            }

            angular.forEach(buttons,function(v){
                var id = v.split('-')[1];
                angular.element($element).find('li').each(function(){
                    if(angular.element(this).find('button').hasClass('btn-'+id)){
                        vm.elements[id]={$el:angular.element(this),id:id}
                        // order buttons according to markup
                        angular.element(this).show();
                        angular.element($element).find('ul').append(angular.element(this))
                    }
                });
            });

            angular.forEach(vm.elements,function(obj){
                obj.$el.on('click',function(event){
                    event.preventDefault();
                    if(vm.resource){
                        AppService.scope.$broadcast('crudButtonClick',{action:obj.id,ID:vm.resource.ID,resource:vm.resource,bundle:vm.resource.bundle})
                    }
                    if(!vm.resource){
                        AppService.scope.$broadcast('crudButtonClick',{action:obj.id})
                    }
                    if(obj.id === 'actions'){
                        $scope.openActionsModal();
                    }
                });
            });

        };

        $scope.openActionsModal = function(){
            ViewService.openModal({
                template:'app/components/app-base/directives/at-actions/at-actions-modal.html',
                data:{resource:vm.resource,bundle:vm.bundle},
                size:'md'
            });
        };
    }
})();