(function () {
    'use strict';
    angular
        .module('appClient')
        .directive('atGridView', atGridView);

    /** @ngInject */
    function atGridView($animate,$interval,$timeout,Api) {
        var bindings = {
            options: '='
        };
        var directive = {
            restrict: 'E',
            // scope: true,
            controller: AtGridViewController,
            controllerAs: 'vm',
            bindToController: bindings,
            link: function(scope, element, attrs) {}
        };
        return directive;
    };

    /** @ngInject */
    function AtGridViewController($scope,$location,$window,$q,$timeout,Api,AppService,ViewService,Restangular) {
        var vm = this;
        vm.id = AppService.getUUID();

        this.init = function(event){
            if(!vm.options){return;}
            vm.options.scope = $scope;

           // if(!vm.options.vm.gridView ){
            vm.gridView = vm.options.vm.gridView = new CrudView(vm.options);
           // }
            vm.gridView.init();
            if(vm.options.gridConfig){
                if(vm.gridApi) {
                    vm.gridView.parseGrid();
                }else{
                    $scope.$watch('vm.options.scope.gridApi',function(){
                        if(!vm.options.scope.gridApi){return;}
                        vm.gridView.initGridApi(vm.options.scope.gridApi);
                    });
                }
            }
        };
        $scope.$watch('vm.options',this.init);

        function CrudView(options){
            var _this = this;
            this.options = options;
            this.id = AppService.getUUID();
            this.options.vm.lovs = Api.getLovItems;

            this.init = function(){
                var base = options.bundle;

                this.baseResource = this.options.baseResource = Restangular.all(Api.model.routes['/' + base]);

                if(this.options.bundleParent){
                    this.baseResource = this.options.baseResource = Restangular.all(Api.model.routes['/' + options.bundleParent]+'/'+this.options.idParent+'/'+options.bundle);
                }

                if(this.options.gridConfig){
                    var gridConfig = this.options.gridConfig;
                    gridConfig.scope = this.options.scope;
                    if(!gridConfig.rowSelectHandler) {
                        this.rowSelectHandler = gridConfig.rowSelectHandler = this.onRowSelect;
                    }else{
                        this.rowSelectHandler = gridConfig.rowSelectHandler;
                    }
                    if(!_this.grid){
                        _this.grid = _this.options.vm.grid = new ViewService.Grid(gridConfig);
                    }
                    this.getList().then(function(res){
                        _this.grid.options.data = res;
                        if($location.search()[_this.options.bundle]){
                            _this.selectRowById($location.search()[_this.options.bundle]);
                        };
                    });
                }

            };

            this.getList = function(){
                var defer = $q.defer();
                var bundle = this.options.bundle;
                var queryParams = {};
                var fields = 'ID,';
                if(this.options.idField){
                    fields += this.options.idField + ',';
                }
                angular.forEach(this.options.gridConfig.columnDefs,function(col){
                    fields += col.field + ',';
                });
                queryParams.fields = fields;
                if (this.options.filter) {
                    if(this.options.vm){
                        this.options.vm.currentFilter = this.options.filter;
                    }
                    queryParams.mainFilter = this.options.filter;
                }
                if (this.options.query) {
                    angular.forEach(this.options.query, function (val, key) {
                        queryParams[key] = val;
                    });
                }
                this.baseResource.getList(queryParams).then(function(res){
                    AppService.scope.$broadcast('list-loaded',{ response:res,bundle:bundle});
                    defer.resolve(res);
                    $timeout(function(){
                        $window.dispatchEvent(new Event('resize'));
                    },1500)
                }).catch(Api.errorHandler);
                return defer.promise;
            };

            this.getResource = function(id){
                var defer = $q.defer();
                this.setQueryParams(id)
                Api.getResource(_this.baseResource,id,this.options.bundle).then(function(res){
                    _this.resource = res;
                    _this.options.vm.resource = res;
                    defer.resolve(res);
                });
                return defer.promise;
            };

            this.setQueryParams = function(id){
                var params = { };
                // primary
                params[_this.options.bundle] = id;
                // secondary
                if (_this.options.bundleParent) {
                    params[_this.options.bundleParent] = _this.options.idParent;
                }
                $location.search(params);
            };

            this.onRowSelect = function(gridRow){
                ViewService.checkFormChanged().then(function(bool){
                    if(bool===false) {
                        // unselect
                        if(gridRow.isSelected === false){
                            _this.resource = undefined;
                            _this.options.vm.resource = undefined;
                            return;
                        }
                        // select
                        var id = gridRow.entity.ID;
                        if(_this.options.idField){
                            id = gridRow.entity[_this.options.idField];
                        }
                        // bugfix: reset secondary
                        if(!_this.options.bundleParent){
                            _this.options.vm.secondary = undefined;
                            _this.options.vm.bundleSecondary = undefined;
                        }
                        _this.getResource(id);
                    }else{
                        return;
                    }
                });
            };

             this.selectRowById = function(id){
                _this.grid.selectById(id);
                 //_this.grid.scrollToFocus(_this.options.data[index])
             };

             this.initGridApi = function(gridApi){
                 _this.gridApi = vm.gridApi = gridApi;
                _this.parseGrid();
             };

             this.parseGrid = function(){
                 _this.gridApi = vm.gridApi;
                 _this.grid.gridApi = vm.gridApi;
                 vm.gridOptions = _this.grid.options;
                 if(vm.gridApi.listeners.length < 1){
                    vm.gridApi.selection.on.rowSelectionChanged(options.scope,_this.rowSelectHandler);
                 }
                $timeout(function(){
                    $window.dispatchEvent(new Event('resize'));
                },1600)
            }

        }

    }
})();
