(function () {
    'use strict';
    
    angular
        .module('appClient')
        .service('Api', Api);

    /** @ngInject */
    function Api(at,$q,$confirm,$http,$cookieStore,$filter,$log,$location,_,toastr,AppService,AppSettings,Restangular) {
        var service = {};
        service.model = {
            permissionsObject: {},
            permission: {},
            permissionDefault: {},
            routes: {},
            lovs: {},
            entities: {},
            forms: {}
        };
        service.init = init;
        service.createResource = createResource;
        service.postResource = postResource;
        service.getResource = getResource;
        service.putResource = putResource;
        service.deleteResource = deleteResource;
        service.getEmbeddedResources = getEmbeddedResources;
        service.errorHandler = errorHandler;
        service.initRoutes = initRoutes;
        service.checkCookieCredentials = checkCookieCredentials;
        service.clearCredentials = clearCredentials;
        service.logOut = logOut;
        service.loadContextualData = loadContextualData;
        service.loadLovs = loadLovs;
        service.getLovItems = getLovItems;
        service.checkAlias = checkAlias;
        service.loadPermissions = loadPermissions;
        service.parsePermissions = parsePermissions;
        service.getPermission = getPermission;
        service.setPermission = setPermission;
        service.restorePermission = restorePermission;
        service.restorePermissions = restorePermissions;
        service.restoreBundlePermissions = restoreBundlePermissions;
        service.routes = service.model.routes;
        service.forms = service.model.forms;
        return service;

        function init(){
            var defer = $q.defer();

            service.initRoutes();
            service.checkCookieCredentials();

            // has valid cookie
            if(AppSettings.currentUser) {
                service.loadContextualData().then(function () {
                    defer.resolve();
                    AppService.scope.$broadcast('login');
                });
            }else{
                // no valid cookie: show login component
                service.logOut();
            }
            return defer.promise;
        }

        function getResource(baseResource,itemId,bundle){
            var defer = $q.defer();
            baseResource.one(itemId).get().then(function(res){
                getEmbeddedResources(res).then(function (res){
                    if (res.bundle == undefined) {
                        res.bundle = bundle;
                    }
                    res._session.state = 'view';
                    service.model.entities[itemId] = AppService.entities[itemId] = res;
                    defer.resolve(res);
                    AppService.scope.$broadcast('resource-loaded',{resource:res});
                });
            }).catch(service.errorHandler);
            return defer.promise;
        }

        function postResource(baseResource,resourceData){
            if(AppSettings.dev){
                console.log('postResource');
                console.log(baseResource);
            }
            var defer = $q.defer();
            // strip resource _session
            var embeddedResourcesCopy;
            AppService.scope.$broadcast('pre-save-resource',{resource:resourceData});
            if(resourceData._session){
                embeddedResourcesCopy = angular.copy(resourceData._session);
                resourceData._session = undefined;
            }
            baseResource.post(resourceData).then(function(res) {
                if(embeddedResourcesCopy){
                    res._session = embeddedResourcesCopy;
                    res._session.state = 'view';
                }
                defer.resolve(res);
            }).catch(service.errorHandler);
            
            AppService.scope.$on('formError',function(event,data){
                if(resourceData){
                    resourceData._session = embeddedResourcesCopy;
                }
            });
            return defer.promise;
        }

        function putResource(resource){
            var defer = $q.defer();
            // strip embedded _session
            var embeddedResourcesCopy;
            AppService.scope.$broadcast('pre-save-resource',{resource:resource});
            if(resource._session){
                embeddedResourcesCopy = angular.copy(resource._session);
                resource._session = undefined;
            }
            // put resource
            resource.put().then(function(res){
                if(embeddedResourcesCopy){
                    resource._session = embeddedResourcesCopy;
                    resource._session.state = 'view';
                }
                AppService.scope.$broadcast('save-item',{resource:resource});
                defer.resolve(res);
            }).catch(service.errorHandler);
            
            AppService.scope.$on('formError',function(event,data){
                if(resource){
                    resource._session = embeddedResourcesCopy;
                }
            });
            return defer.promise;
        }

        function deleteResource(resource){
            var defer = $q.defer();
            $confirm({item:resource},{ templateUrl: 'app/components/delete-dialog/delete-dialog.html' })
                .then(function() {
                    AppService.scope.$broadcast('delete-item',{ resource:resource });
                    defer.resolve();
                });
            return defer.promise;
        }

        function createResource(bundle){
            var defer = $q.defer();
            if(!AppSettings.api_structure[bundle]){
                $log.error('api_structure for '+bundle+ ' is not defined in appsettings.json')
                return defer.promise;
            }
            var resourceDefaults = angular.toJson(AppSettings.api_structure[bundle]);
            var resource = angular.fromJson(resourceDefaults);
            resource._session = {};
            resource.bundle = bundle;
            AppService.scope.$broadcast('init-resource',{resource:resource});
            getEmbeddedResources(resource).then(function(res){
                res._session.state = 'create';
                AppService.scope.$broadcast('create-resource',{resource:res});
                defer.resolve(res);
            });
            return defer.promise;
        }

        function getEmbeddedResources(item){
            var defer = $q.defer();
            var count = 0;
            var embeddedResources = [];
            if(!item._session){
                item._session = {};
            }
            _.each(item,function(v,k){
                if (v !== null && typeof v === 'object' && k !== "parentResource") {
                    var route = service.model.routes["/" + k];
                    if (route !== undefined) {
                        embeddedResources.push({ key: k, value: route });
                    }
                }
                if(k.match('_resource') && item[k]){
                    embeddedResources.push({key:k,value:v});
                }
            });
            if(embeddedResources.length) {
                _.each(embeddedResources, function (resource) {
                    Restangular.all(resource.value).getList().then(function (res) {
                        item._session[resource.key] = res;
                        count++;
                        if (count === embeddedResources.length) {
                            defer.resolve(item);
                        }
                    }, function (error) {
                        console.log(error);
                        item._session[resource.key] = [];
                        count++;
                        if (count === embeddedResources.length) {
                            defer.resolve(item);
                        }
                    }
                    
                    );
                });
            }else{
                defer.resolve(item);
            }
            return defer.promise;
        }

        function initRoutes(){
            Restangular.setBaseUrl(AppSettings.api_host);
            service.model.baseUrl = AppSettings.api_host;
            _.each(AppSettings.api_routes, function(obj){
                _.each(obj,function(v,k){
                    service.model.routes[k] = v;
                });
            });
            if(AppSettings.dev){
                _.each(AppSettings.api_routes_dev, function(obj){
                    _.each(obj,function(v,k){
                        // override production routes for dev
                        service.model.routes[k]=v;
                    });
                });
            }
        }

        function checkCookieCredentials(){
            var loggedIn = $cookieStore.get('globals') || undefined;
            if (loggedIn) {
                AppSettings.currentCredentials = $cookieStore.get('globals').currentCredentials;
                AppSettings.currentUser = $cookieStore.get('globals').currentUser;
                $http.defaults.headers.common['Authorization'] = 'Bearer ' + $cookieStore.get('globals').token;
            }
        }

        function loadContextualData(){
            var defer = $q.defer();
            service.loadPermissions().then(service.loadLovs).then(defer.resolve);
            return defer.promise;
        }

        function loadPermissions(){
            var defer = $q.defer();
            var basePerms =  Restangular.all(service.model.routes["/permissies"]);
            basePerms.get('').then(function(res){
                service.model.permissionsObject = res[0];
                parsePermissions(res[0]);
                defer.resolve();
            }).catch(service.errorHandler);
            return defer.promise;
        }

        function parsePermissions(perms){
            var permissions = AppService.copy(perms);
            _.each(permissions.primair,function(permission,key){
                var crud = service.model.permissionDefault[permission.table] = [];
                crud[0] = permission.permissionCreate;
                crud[1] = permission.permissionRead;
                crud[2] = permission.permissionUpdate;
                crud[3] = permission.permissionDelete;
                if(permission.secundair && permission.secundair.length){
                    _.each(permission.secundair,function(v,k){
                        var crud2 = service.model.permissionDefault[permission.table+'.'+ v.attribute] = [];
                        crud2[0] = v.permissionCreate;
                        crud2[1] = v.permissionRead;
                        crud2[2] = v.permissionUpdate;
                        crud2[3] = v.permissionDelete;
                    });
                }
            });
            service.model.permission = AppService.copy(service.model.permissionDefault);
            if(AppSettings.dev)
            console.log(service.model.permissionDefault);
        }

        function getPermission(identifier){
           if(!identifier){
               console.log('getPermission(): identifier='+identifier);
               return false;
           }
            var crud = [true,true,true,true];
            if(service.model.permission[identifier]){
                crud = service.model.permission[identifier]
            }
            return crud;

            // var primair = identifier.split('.')[0];
            // var secundair = identifier.split('.')[1];
            // var crud;
            // _.each(service.model.permissionsObject.primair,function(value,key){
            //     if(value.table===primair){
            //         if(!secundair || !value.secundair.length) {
            //             crud = crudArray(value)
            //         }else{
            //             _.each(value.secundair,function(value2,key2){
            //                 if(value2.attribute===secundair){
            //                     //console.log(secundair);
            //                     //console.log(value2);
            //                     crud = crudArray(value2)
            //                     //console.log(crud);
            //                 }
            //             })
            //         }
            //     }
            // });
            // if(!crud){
            //     crud = [true,true,true,true];
            // }
            // return crud;
            //
            // function crudArray(value){
            //     var crud = [false,false,false,false];
            //     if(value.permissionCreate){
            //         crud[0]=true;
            //     }
            //     if(value.permissionRead){
            //         crud[1]=true;
            //     }
            //     if(value.permissionUpdate){
            //         crud[2]=true;
            //     }
            //     if(value.permissionDelete){
            //         crud[3]=true;
            //     }
            //     return crud;
            // }
        }

        function setPermission(identifier,crud){
            service.model.permission[identifier] = crud;
            AppService.scope.$broadcast('permissionChange');

            // var primair = identifier.split('.')[0];
            // var secundair = identifier.split('.')[1];
            // var crud;
            // var changed;
            // _.each(service.model.permissionsObject.primair,function(value){
            //     if(value.table===primair){
            //         if(!secundair || !value.secundair.length) {
            //             value.permissionCreate = crud[0];
            //             value.permissionRead = crud[1];
            //             value.permissionUpdate = crud[2];
            //             value.permissionDelete = crud[3];
            //             changed = true;
            //         }else{
            //             // fixme: check in at-crud-form
            //             _.each(value.secundair,function(value2){
            //                 //console.log(value2);
            //                 if(value2.attribute===secundair){
            //                     console.log('crud secundair old: ');
            //                     console.log(value);
            //                     value2.permissionCreate = crud[0];
            //                     value2.permissionRead = crud[1];
            //                     value2.permissionUpdate = crud[2];
            //                     value2.permissionDelete = crud[3];
            //                     changed = true;
            //                     console.log('crud secundair new: ');
            //                     console.log(value);
            //                 }
            //             })
            //         }
            //     }
            // });
            // if(changed){
            //     AppService.scope.$broadcast('permissionChange');
            // }
        }

        function restorePermission(identifier){
            if(service.model.permissionDefault[identifier]){
                service.model.permission[identifier] = AppService.copy(service.model.permissionDefault[identifier]);
                AppService.scope.$broadcast('permissionChange');
            }
            // var primair = identifier.split('.')[0];
            // var secundair = identifier.split('.')[1];
            // _.each(service.model.permissionsObject.primair,function(value,key){
            //     if(value.table===primair){
            //         if(!secundair || !value.secundair.length) {
            //             var crud = service.model.permissionDefault[primair];
            //             var permission = service.model.permissionsObject.primair;
            //             permission[key].permissionCreate = crud[0];
            //             permission[key].permissionRead = crud[1];
            //             permission[key].permissionUpdate = crud[2];
            //             permission[key].permissionDelete = crud[3];
            //         }else{
            //             _.each(value.secundair,function(value2,key2){
            //                 if(value2.attribute===secundair){
            //                     var crud =  service.model.permissionDefault[primair+'.'+secundair];
            //                     var permission = service.model.permissionsObject.primair[key]['secundair'];
            //                     permission[key2].permissionCreate = crud[0];
            //                     permission[key2].permissionRead = crud[1];
            //                     permission[key2].permissionUpdate = crud[2];
            //                     permission[key2].permissionDelete = crud[3];
            //                 }
            //             })
            //         }
            //     }
            // });
            // AppService.scope.$broadcast('permissionChange');
        }

        function restoreBundlePermissions(resource){
            service.restorePermission(resource.bundle);
            var object;
            if(!resource.ID) {
                object = resource;
            }else{
                object = resource.plain();
            }
            _.each(object,function(value,key) {
                if(service.model.permissionDefault[resource.bundle] && service.model.permissionDefault[resource.bundle+'.'+key]){
                    service.restorePermission[resource.bundle+'.'+key];
                }
            });
        }

        function restorePermissions(){
            _.each(service.model.permissionDefault,function(value,key){
                service.model.permission[key] = AppService.copy(value);
            })
        }

        function loadLovs(){
            var defer = $q.defer();
            var baseLovs =  Restangular.all(service.model.routes["/lovs"]);
            baseLovs.get('').then(function(res){
                service.model.lovs = res;
                defer.resolve();
            }).catch(service.errorHandler);
            return defer.promise;
        }

        function getLovItems(tab){
            var arr = [];
            _.each(service.model.lovs,function(lov,k){
               if(lov.tab === tab){
                   arr.push(lov);
               }
            });
            return arr;
        }

        function checkAlias(tag,bundle){
            var defer = $q.defer();
            var aliasApi =  Restangular.all(service.model.routes["/"+bundle+"Alias"]);
            aliasApi.get('',{aliasCode: encodeURI(tag.text)}).then(function(res){
                if(res.valid === true) {
                    defer.resolve(true);
                }else{
                    toastr.warning($filter('translate')(res.reason)+'.');
                    defer.resolve(false);
                }
            }).catch(service.errorHandler);
            return defer.promise;
        }

        function logOut(refresh){
            service.clearCredentials();
            AppSettings.ready = true;
            if (refresh) {
                $location.href = location.origin;
            } else {
                $location.search({});
                $location.path('/');            	
            }
            $log.log('api.logOut()');
        }

        function clearCredentials() {
            $cookieStore.remove('globals');
            //$http.defaults.headers.common.Authorization = 'Basic';
            AppSettings.currentCredentials = undefined;
            AppSettings.currentUser = undefined;
            AppSettings.loggedIn = false;
        }

        function errorHandler(error){
            AppService.scope.$broadcast('formError', error);
            if(AppSettings.dev){
                console.log(error);
            }
            if(!error){
                return;
            }
            if(error.data === "token expired"){
                toastr.warning($filter('translate')('LOGIN EXPIRED')+'.');
                service.logOut();
                return;
            }
            if (error.statusText === 'Bad Request' || error.status == 400) {
                if (error.data.message) {
                    toastr.warning($filter('translate')(error.data.message));
                } else {
                    toastr.warning($filter('translate')('ERROR BAD REQUEST') + '.');
                }
            }
            if(error.statusText === 'Unauthorized' || error.status == 401){
                toastr.warning($filter('translate')('UNAUTHORIZED ERROR')+'.');
                service.logOut();
                return;
            }
            if(error.statusText === 'Not found' || error.status == 404){
                toastr.warning($filter('translate')('ERROR NOT FOUND')+'.');
            }
        }

    }
 
})();