(function () {
    'use strict';

    angular
        .module('appClient')
        .config(function (valdrProvider) {
            valdrProvider.addValidator('bsn');
            valdrProvider.addValidator('btw');
            valdrProvider.addValidator('currency');
            valdrProvider.addValidator('date');
            valdrProvider.addValidator('dateCompare');
            valdrProvider.addValidator('dateBetween');
            valdrProvider.addValidator('iban');
            valdrProvider.addValidator('initials');
            valdrProvider.addValidator('integer');
            valdrProvider.addValidator('number');
            valdrProvider.addValidator('phone');
            valdrProvider.addValidator('password');
            valdrProvider.addValidator('zipcode');
        })

        /** @ngInject */
        .factory('bsn', function (ValidationConstants) {
            function isBsn(bsn){
                if(!bsn){ return false;}
                var j = bsn.length;
                if (j<8 || j>9 || ValidationConstants.REGEX_INTEGER.test(bsn) == false) {
                    return false;
                }
                var pos = 0;
                var result = 0;
                for (var i=j; i>0; i--) {
                    result += (i != 1) ? (bsn.charAt(pos) * i) : (bsn.charAt(pos) * i * -1);
                    pos++;
                }
                return (result % 11 == 0);
            }
            return {
                name: 'bsn',
                validate: function (value) {
                    if(value != '') {
                        return isBsn(value);
                    }else{
                        return true;
                    }
                }
            };
        })
        /** @ngInject */
        .factory('btw', function (ValidationConstants) {
            return {
                name: 'btw',
                validate: function (value) {
                    if(value != '') {
                        return ValidationConstants.REGEX_BTW_NL.test(value);
                    }else{
                        return true;
                    }
                }
            };
        })
        /** @ngInject */
        .factory('currency', function (ValidationConstants) {
            return {
                name: 'currency',
                validate: function (value) {
                    if(value != '') {
                        return ValidationConstants.REGEX_CURRENCY_EURO.test(value);
                    }else{
                        return true;
                    }
                }
            };
        })
        /** @ngInject */
        .factory('date', function (ValidationConstants) {
            return {
                name: 'date',
                validate: function (value) {
                    if(value != '') {
                        return ValidationConstants.REGEX_DATE.test(value);
                    }else{
                        return true;
                    }
                }
            };
        })
        /** @ngInject */
        .factory('dateCompare', function (ValidationConstants,$filter,AppSettings,moment) {
            return {
                name: 'dateCompare',
                validate: function (value, valdrArguments) {
                    if(value != '') {
                        var resource = AppSettings.currentResource;
                        var operator = valdrArguments.operator;
                        var valid = true;
                        var message = '';
                        var tr = $filter('translate');

                        // date field comparison
                        if(valdrArguments.compareToField){
                            var compareToField = valdrArguments.compareToField;
                            var a = moment(value, AppSettings.dateFormat);
                            var b = moment(resource[compareToField], AppSettings.dateFormat);
                            if (operator === '>' && !(a > b)) {
                                message = tr('Deze datum mag niet eerder of gelijk dan') + ' ' + resource[compareToField] + ' ' + tr('zijn');
                                valid = false;
                            }
                            if (operator === '>=' && !(a >= b)) {
                                message = tr('Deze datum mag niet eerder dan') + ' ' + resource[compareToField] + ' ' + tr('zijn');
                                valid = false;
                            }
                            if (operator === '<' && !(a < b)) {
                                message = tr('Deze datum moet eerder dan') + ' ' + resource[compareToField] + ' ' + tr('zijn');
                                valid = false;
                            }
                            if (operator === '<=' && !(a <= b)) {
                                message = tr('Deze datum moet eerder of gelijk aan') + ' ' + resource[compareToField] + ' ' + tr('zijn');
                                valid = false;
                            }
                            if (operator === '=' || operator === '==' || operator === '===') {
                                if (!(value === resource[compareToField])) {
                                    message = tr('Deze datum moet gelijk aan') + ' ' + resource[compareToField] + ' ' + tr('zijn');
                                    valid = false;
                                }
                            }
                        }

                        // date time unit comparison
                        if(valdrArguments.unit && valdrArguments.amount){
                            var a = moment(value,AppSettings.dateFormat);
                            var b;
                            var amount = valdrArguments.amount;
                            if(!valdrArguments.reference || valdrArguments.reference === 'now') {
                                b = moment();
                            }else{
                                b = moment(valdrArguments.reference, AppSettings.dateFormat);
                            }
                            var diff = b.diff(a,valdrArguments.unit);
                            var diffFormatted = b.subtract(valdrArguments.amount, valdrArguments.unit).format(AppSettings.dateFormat);

                            if (operator === '>' && !(amount > diff)) {
                                message = tr('Deze datum mag niet eerder of gelijk dan') + ' ' + diffFormatted + ' ' + tr('zijn');
                                valid = false;
                            }
                            if (operator === '>=' && !(amount >= diff)) {
                                message = tr('Deze datum mag niet eerder dan') + ' ' + diffFormatted + ' ' + tr('zijn');
                                valid = false;
                            }
                            if (operator === '<' && !(amount < diff)) {
                                message = tr('Deze datum moet eerder dan') + ' ' + diffFormatted + ' ' + tr('zijn');
                                valid = false;
                            }
                            if (operator === '<=' && !(amount <= diff)) {
                                message = tr('Deze datum moet eerder of gelijk aan') + ' ' + diffFormatted + ' ' + tr('zijn');
                                valid = false;
                            }
                            if (operator === '=' || operator === '==' || operator === '===') {
                                if (!(a === b)) {
                                    message = tr('Deze datum moet gelijk aan') + ' ' + diffFormatted + ' ' + tr('zijn');
                                    valid = false;
                                }
                            }
                        }
                        valdrArguments.message = message;
                        if(valdrArguments.messageOverride){
                            valdrArguments.message = valdrArguments.messageOverride;
                        }
                        return valid;
                    }else{
                        return true;
                    }
                }
            }
        })
        /** @ngInject */
        .factory('dateBetween', function (ValidationConstants,$filter,AppSettings,moment) {
            return {
                name: 'dateBetween',
                validate: function (value, valdrArguments) {
                    if(value != '') {
                        var resource = AppSettings.currentResource;
                        var operator = valdrArguments.operator;
                        var valid = true;
                        var message = '';
                        var tr = $filter('translate');

                        var date = moment(value, AppSettings.dateFormat);

                        var fields = valdrArguments.compareToFields.split(',');
                        var begin = moment(resource[fields[0]], AppSettings.dateFormat);
                        var end = moment(resource[fields[1]], AppSettings.dateFormat);
                        var beginFormatted = begin.format(AppSettings.dateFormat);
                        var endFormatted = end.format(AppSettings.dateFormat);

                        if (operator === '==,==') {
                            if(date < begin || date > end){
                                message = tr('Deze datum moet tussen') + ' ' + beginFormatted + ' ' + tr('en') + ' ' +endFormatted + ' ' + tr('liggen');
                                valid = false;
                            }
                        }
                        // todo:
                        // if (operator === '>=,<=') {
                        //     if(date <= begin || date >= end){
                        //         message = tr('Deze datum moet eerder dan') + ' ' + beginFormatted + ' ' + tr('en later dan') + ' ' +endFormatted + ' ' + tr('liggen');
                        //         valid = false;
                        //     }
                        // }
                        valdrArguments.message = message;
                        if(valdrArguments.messageOverride){
                            valdrArguments.message = valdrArguments.messageOverride;
                        }
                        // pass test when the compared date fields are invalid or empty strings
                        if(message.match('Invalid date')){
                            valid = true;
                        }
                        return valid;
                    }else{
                        return true;
                    }
                }
            }
        })
        /** @ngInject */
        .factory('iban', function (ValidationConstants) {
            return {
                name: 'iban',
                validate: function (value) {
                    if(value != '') {
                        return ValidationConstants.REGEX_IBAN.test(value);
                    }else{
                        return true;
                    }
                }
            };
        })
        /** @ng /** @ngInject */
        .factory('initials', function (ValidationConstants) {
            return {
                name: 'initials',
                validate: function (value) {
                    if(value != '') {
                        return ValidationConstants.REGEX_INITIALS.test(value);
                    }else{
                        return true;
                    }
                }
            };
        })
        /** @ngInject */
        .factory('integer', function (ValidationConstants) {
            return {
                name: 'integer',
                validate: function (value) {
                    if(value != '') {
                        return ValidationConstants.REGEX_INTEGER.test(value);
                    }else{
                        return true;
                    }
                }
            };
        })
        /** @ngInject */
        .factory('number', function (ValidationConstants) {
            return {
                name: 'number',
                validate: function (value) {
                    if(value != '') {
                        return ValidationConstants.REGEX_NUMERIC.test(value);
                    }else{
                        return true;
                    }
                }
            };
        })
        /** @ngInject */
        .factory('phone', function (ValidationConstants) {
            return {
                name: 'phone',
                validate: function (value) {
                    if(value != '') {
                        return ValidationConstants.REGEX_PHONE.test(value);
                    }else{
                        return true;
                    }
                }
            };
        })
        /** @ngInject */
        .factory('password', function (ValidationConstants) {
            return {
                name: 'password',
                validate: function (value) {
                    if(value != '') {
                        return ValidationConstants.REGEX_PASSWORD.test(value);
                    }else{
                        return true;
                    }
                }
            };
        })
        /** @ngInject */
        .factory('zipcode', function (ValidationConstants) {
            return {
                name: 'zipcode',
                validate: function (value) {
                    if(value == null){
                        return true;
                    }
                    if(value != '') {
                        return ValidationConstants.REGEX_ZIPCODE.test(value);
                    }else{
                        return true;
                    }
                }
            };
        })

        .constant('ValidationConstants', {
            REGEX_BSN: /^[0-9]{9}$/,
            REGEX_BTW_NL: /(NL)?[0-9]{9}B[0-9]{2}/,
            REGEX_CURRENCY_EURO: /^\s*-?((\d{1,3}(\.(\d){3})*)|\d*)(,\d{1,2})?\s?(\u20AC)?\s*$/,
            REGEX_DATE: /(((0[1-9]|[12][0-9]|3[01])([-])(0[13578]|10|12)([-])([1-2][0,9][0-9][0-9]))|(([0][1-9]|[12][0-9]|30)([-])(0[469]|11)([-])([1-2][0,9][0-9][0-9]))|((0[1-9]|1[0-9]|2[0-8])([-])(02)([-])([1-2][0,9][0-9][0-9]))|((29)(\.|-|\/)(02)([-])([02468][048]00))|((29)([-])(02)([-])([13579][26]00))|((29)([-])(02)([-])([0-9][0-9][0][48]))|((29)([-])(02)([-])([0-9][0-9][2468][048]))|((29)([-])(02)([-])([0-9][0-9][13579][26])))/,
            REGEX_EMAIL: /^[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?$/i,
            REGEX_IBAN: /[a-zA-Z]{2}[0-9]{2}[a-zA-Z0-9]{4}[0-9]{7}([a-zA-Z0-9]?){0,16}/,
            REGEX_INITIALS: /^([a-zA-Z]|[cC][hH]|[cC][Hh][Rr]|[fF][Rr]|[Ii][jJ]|[jJ][aA][Cc]|[pP][Hh]|[tT][Hh]|[tT][Jj])((\.([a-zA-Z]|[cC][hC]|[cC][Hh][Rr]|[fF][Rr]|[Ii][jJ]|[jJ][aA][Cc]|[pP][Hh]|[tT][Hh]|[tT][Jj]))*)(\.)?$/,
            REGEX_ZIPCODE: /^[1-9][0-9]{3} ?(?!sa|sd|ss)[a-z]{2}$/i,
            REGEX_PHONE: /^([+0]{1})([0-9]{9,14})$/i,
            REGEX_PASSWORD:  /^(?=.*[A-Z])(?=.*[!@#$&*])(?=.*[0-9])(?=.*[a-z].*[a-z].*[a-z]).{8,24}$/,
            //(/^
            //(?=.*\d)                //should contain at least one digit
            //(?=.*[a-z])             //should contain at least one lower case
            //(?=.*[A-Z])             //should contain at least one upper case
            //[a-zA-Z0-9]{8,24}         //should contain at least 8 and max 24 from the mentioned characters
            //$/)
            REGEX_URL: /^(?:(?:https?|ftp):\/\/)(?:\S+(?::\S*)?@)?(?:(?!10(?:\.\d{1,3}){3})(?!127(?:\.\d{1,3}){3})(?!169\.254(?:\.\d{1,3}){2})(?!192\.168(?:\.\d{1,3}){2})(?!172\.(?:1[6-9]|2\d|3[0-1])(?:\.\d{1,3}){2})(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))|(?:(?:[a-z\u00a1-\uffff0-9]+-?)*[a-z\u00a1-\uffff0-9]+)(?:\.(?:[a-z\u00a1-\uffff0-9]+-?)*[a-z\u00a1-\uffff0-9]+)*(?:\.(?:[a-z\u00a1-\uffff]{2,})))(?::\d{2,5})?(?:\/[^\s]*)?$/i,
            REGEX_LASTNAME: /^[^0-9]+$/i,
            REGEX_INTEGER: /^[-+]?\d+$/,
            REGEX_ALPHA: /^[a-zA-ZàáâäãåąćęèéêëìíîïłńòóôöõøùúûüÿýżźñçčšžÀÁÂÄÃÅĄĆĘÈÉÊËÌÍÎÏŁŃÒÓÔÖÕØÙÚÛÜŸÝŻŹÑßÇŒÆČŠŽ∂ð ,.'-]+$/,
            REGEX_ALPHANUM: /^[a-zA-Z0-9àáâäãåąćęèéêëìíîïłńòóôöõøùúûüÿýżźñçčšžÀÁÂÄÃÅĄĆĘÈÉÊËÌÍÎÏŁŃÒÓÔÖÕØÙÚÛÜŸÝŻŹÑßÇŒÆČŠŽ∂ð ,.'-]+$/,
            REGEX_NUMERIC: /^[0-9]+$/,
            REGEX_NO_SPACES: /^\S+$/,
            REGEX_NO_PUNCTUATION: /^[^,.'-]+$/,
        })

})();