(function () {
    'use strict';
    
    angular
        .module('appClient')
        .service('ViewService', ViewService)
        .controller('ModalInstanceController', ModalInstanceController);
    
    /** @ngInject */
    function ViewService(_, $q, $timeout, $compile, $confirm, $filter, $document, $translate, $http, $templateCache, AppService, Api, AppSettings , $uibModal, uiGridConstants, localStorageService) {
        var service = {};
        service.Grid = Grid;
        service.bootstrapView = bootstrapView;
        service.openModal = openModal;
        service.getListViewColumns = getListViewColumns;
        service.setListViewLovFilter = setListViewLovFilter;
        service.checkFormChanged = checkFormChanged;
        service.fixPaginationLinks = fixPaginationLinks;
        service.template = template;
        service.getStyle = getStyle;
        service.scrollToElement = scrollToElement;
        service.modals = {};
        return service;
        
        function Grid(options) {
            var _this = this;
            this.options = options;
            this.id = AppService.getUUID();
            this.defaults = {
                // data: {},
                enableFiltering: true,
                enableSorting: true,
                exporterMenuPdf: false,
                exporterMenuCsv: false,
                enableGridMenu: true,
                enableRowSelection: true,
                enableFullRowSelection: true,
                enableColumnResizing: true,
                enableSelectAll: false,
                noUnselect: false,
                enableHorizontalScrollbar: uiGridConstants.scrollbars.WHEN_NEEDED,
                // enableVerticalScrollbar: uiGridConstants.scrollbars.NEVER,
                selectionRowHeaderWidth: 31,
                rowHeight: 31,
                showGridFooter: true,
                onRegisterApi: function (gridApi) {
                    options.scope.gridApi = _this.gridApi = gridApi;
                    // gridApi.selection.on.rowSelectionChanged(options.scope,options.rowSelectHandler);
                    gridApi.selection.setMultiSelect(false);
                    gridApi.selection.clearSelectedRows();
                    //var cellTemplate = 'ui-grid/selectionRowHeader';   // you could use your own template here
                    //$scope.gridApi.core.addRowHeaderColumn( { name: 'rowHeaderCol', displayName: '', width: 30, cellTemplate: cellTemplate} );
                    if (options.selectedIndex === 0 || options.selectedIndex) {
                        _this.selectRow(options.selectedIndex);
                    }
                }
            };
            // override grid defaults
            angular.forEach(this.defaults, function (v, k) {
                if (!options[k]) {
                    options[k] = v;
                }
            });
            
            this.selectRow = function (index) {
                $timeout(function () {
                    if (_this.gridApi) {
                        if (_this.gridApi.selection.selectRow) {
                            if (_this.options.data[index]) {
                                // todo: scrollToFocus(rowEntity, colDef)
                                if(_this.gridApi.cellNav){
                                    _this.gridApi.cellNav.scrollToFocus(_this.options.data[index]);
                                }
                                _this.gridApi.selection.selectRow(_this.options.data[index]);
                            }
                        }
                    }
                });
            };
            
            this.selectById = function (id) {
                if (_this.gridApi) {
                    if (_this.gridApi.selection.selectRow) {
                        angular.forEach(_this.options.data, function (v, k) {
                            if (v.ID === id) {
                                _this.selectRow(k);
                            }
                        });
                    }
                }
            }
        }
        
        /*
        // view helper functions
        */
        function bootstrapView(callback) {
            AppSettings.currentForm = undefined;
            if (!AppSettings.ready) {
                AppService.scope.$on('init-app', function(){
                    callback.call();
                });
            } else {
                callback.call();
            }
        }
        
        function openModal(options) {
            var controller = options.controller || ModalInstanceController;
            var modalInstance = $uibModal.open({
                animation: true,
                templateUrl: options.template,
                controller: controller,
                size: options.size,
                resolve: {
                    data: function () {
                        return options.data;
                    }
                }
            });
            AppSettings.currentModalController = controller;
            modalInstance.result.then(function (data) {
                console.log('Modal ok.');
                //console.log(data);
            }, function (data) {
               // console.log('Modal cancel.');
                //console.log(data);
            });
        }

        function scrollToElement(id,offset,duration){
            var element = angular.element(document.getElementById(id));
            var yOffset = offset + AppSettings.headerHeight + 70 || AppSettings.headerHeight + 70;
            var time = duration || 600;
            $document.scrollToElement(element, yOffset, time);
        }
        
        function getListViewColumns(bundle) {
            var columns = [];
            angular.forEach(AppSettings.listViews[bundle], function (col) {
                var displayName = $translate.instant(bundle + '.' + col.field);
                if(displayName === bundle + '.' + col.field){
                    displayName = $translate.instant(col.field);
                }
                var crud = Api.getPermission(bundle + '.' + col.field);
                if (crud[1] !== false) {
                    var column = {
                        field: col.field,
                        displayName: displayName
                    };
                    if (col.width) {
                        column.width = col.width;
                    }
                    if (col.visible == false) {
                        column.visible = col.visible;
                    }
                    columns.push(column);
                }
            });
            return columns;
        }

        function setListViewLovFilter(options) {
            var lovs = Api.getLovItems(options.lovTable);
            var lovOptions = [];
            var term = options.default || undefined;
            angular.forEach(lovs, function (lov) {
                lovOptions.push({value: lov.description, label: lov.description});
            });
            options.col.filter = {
                term: term,
                type: uiGridConstants.filter.SELECT,
                selectOptions: lovOptions
            };

        }

        function checkFormChanged(){
            var defer = $q.defer();
            if(AppSettings.currentForm && AppSettings.currentForm.$dirty) {
                var msg = 'Uw wijzigingen gaan verloren als u deze actie uitvoert';
                $confirm({
                    text: $filter('translate')(msg) + '.',
                    title: $filter('translate')('WARNING'),
                    ok: $filter('translate')('OK'),
                    cancel: $filter('translate')('CANCEL')
                }).then(function () {
                    AppSettings.currentForm = undefined;
                    defer.resolve(false);
                }, function () {
                    defer.resolve(true);
                });
            } else {
                // AppSettings.currentForm = undefined;
                defer.resolve(false);
            }
            return defer.promise;
        };

        // fix for bootstrap pagination hrefs
        function fixPaginationLinks(){
            $timeout(function(){
                angular.element('body').find('.pagination li a').each(function(el,i){
                    angular.element(this).removeAttr('href');
                })
            },1000)
        }
        
        /*
        // lazy template loading
        //
        // usage inside directives: ViewService.template($element,'app/components/templates/test.html,$scope).then(function(){};
        //
        */
        function template(element, templateUrl, scope) {
            var defer = $q.defer();
            var templateLoader;
            templateLoader = $http.get(templateUrl, { cache: $templateCache });
            var promise = templateLoader.success(function (html) {
                element.html(html);
            }).then(function (response) {
                defer.resolve(scope);
                element.replaceWith($compile(element.html())(scope));
            });
            return defer.promise;
        }

        function getStyle(el, styleProp) {
            var value, defaultView = el.ownerDocument.defaultView;
            // W3C standard way:
            if (defaultView && defaultView.getComputedStyle) {
                // sanitize property name to css notation (hypen separated words eg. font-Size)
                styleProp = styleProp.replace(/([A-Z])/g, "-$1").toLowerCase();
                return defaultView.getComputedStyle(el, null).getPropertyValue(styleProp);
            } else if (el.currentStyle) { // IE
                // sanitize property name to camelCase
                styleProp = styleProp.replace(/\-(\w)/g, function(str, letter) {
                    return letter.toUpperCase();
                });
                value = el.currentStyle[styleProp];
                // convert other units to pixels on IE
                if (/^\d+(em|pt|%|ex)?$/i.test(value)) {
                    return (function(value) {
                        var oldLeft = el.style.left, oldRsLeft = el.runtimeStyle.left;
                        el.runtimeStyle.left = el.currentStyle.left;
                        el.style.left = value || 0;
                        value = el.style.pixelLeft + "px";
                        el.style.left = oldLeft;
                        el.runtimeStyle.left = oldRsLeft;
                        return value;
                    })(value);
                }
                return value;
            }
        }

    }
    
    function ModalInstanceController($scope, $uibModalInstance, data) {
        $scope.data = data;
        $scope.ok = function (data) {
            $uibModalInstance.close(data);
        };
        $scope.close = $scope.ok;
        $scope.cancel = function (data) {
            $uibModalInstance.dismiss(data);
           // $uibModalInstance.dismiss('cancel');
        };
    }
 
})();