"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var util_1 = require("./util/util");
var storage_1 = require("./util/storage");
var Config = /** @class */ (function () {
    function Config() {
        this._config = {};
        this._locked = ['_config', '_defaults'];
    }
    Config.prototype.get = function (key) {
        return this._config[key];
    };
    Config.prototype.set = function (key, value, castToType) {
        if (castToType === void 0) { castToType = false; }
        if (!this.islocked(key)) {
            if (castToType) {
                value = util_1.Util.castToType(value);
            }
            return this._config[key] = value;
        }
        else {
            console.error('Config: property "' + key + '" can not be changed.');
        }
    };
    Config.prototype.add = function (object, extend, castToType) {
        if (extend === void 0) { extend = false; }
        if (castToType === void 0) { castToType = false; }
        var self = this;
        if (!extend) {
            Object.keys(object).forEach(function (key) {
                self.set(key, object[key], castToType);
            });
        }
        else {
            this.extend(object);
        }
    };
    Config.prototype.fetch = function (url, extend) {
        if (extend === void 0) { extend = false; }
        return __awaiter(this, void 0, void 0, function () {
            var self, config;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        self = this;
                        return [4 /*yield*/, fetch(url)
                                .then(function (response) {
                                return response.json();
                            })
                                .catch(function (error) {
                                console.log('config fetch error:', error.message);
                            })];
                    case 1:
                        config = _a.sent();
                        if (extend) {
                            this.extend(config);
                        }
                        else {
                            Object.keys(config).forEach(function (k, i) {
                                self.set(k, config[k]);
                            });
                        }
                        return [2 /*return*/, config];
                }
            });
        });
    };
    Config.prototype.extend = function (object, target) {
        if (target === void 0) { target = this._config; }
        util_1.Util.deepAssign(target, object);
    };
    Config.prototype.all = function () {
        return this._config;
    };
    Config.prototype.getLocalSettings = function (id) {
        if (window.localStorage && window.localStorage.getItem(id)) {
            this.add(storage_1.Storage.restoreLocalModel(id));
        }
    };
    Config.prototype.setLocalSettings = function (id, settings) {
        if (settings === void 0) { settings = this._config; }
        if (window.localStorage) {
            storage_1.Storage.saveLocalModel(settings, id);
        }
    };
    Config.prototype.setDefaults = function (configDefaults) {
        var self = this;
        self._defaults = configDefaults;
        Object.keys(configDefaults).forEach(function (k) {
            self.set(k, util_1.Util.copy(configDefaults[k]));
        });
    };
    Config.prototype.restoreDefaults = function () {
        var self = this;
        Object.keys(self._defaults).forEach(function (k) {
            self.set(k, self._defaults[k]);
        });
    };
    Config.prototype.islocked = function (key) {
        // todo:
        this._locked.forEach(function (k) {
            if (k === key) {
                console.log('Config error: property "' + k + '" is protected');
                return true;
            }
        });
    };
    // todo
    Config.prototype.lock = function (key) {
        // todo:
        return false;
        // this._locked.forEach((k) => {
        //
        // })
    };
    Config.prototype.unLock = function (key) {
        // todo:
        // this._locked.forEach((k) => {
        //
        // })
    };
    Config.prototype.initXdomain = function (object) {
        if (typeof window !== 'undefined' && window['xdomain']) {
            window['xdomain'].masters(object.masters);
            window['xdomain'].slaves(object.slaves);
        }
    };
    ;
    return Config;
}());
exports.Config = Config;
//# sourceMappingURL=config.js.map